<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
//
// Anthill - Copyright (c) 2001-2004 Danen Consulting Services
// http://anthill.vmlinuz.ca/
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// $Id: functions.inc.php,v 1.128 2004/02/13 18:52:30 vdanen Exp $
//
// Anthill functions


/**
 * Handle converting our stored time format to the administrator's 
 * configured default
 *
 * @param $date the time string to convert
 */
function d_getTime($date)
{
    global $_CONF; 
    $year  = substr($date,0,4);
    $mon   = substr($date,4,2);
    $day   = substr($date,6,2);
    $hour  = substr($date,8,2);
    $min   = substr($date,10,2);
    $sec   = substr($date,12,2);
    $gdate = date($_CONF['datefmt'] . " " . $_CONF['timefmt'],mktime($hour,$min,$sec,$mon,$day,$year));
    return($gdate);
}
  
/**
 * Function to set the -f flag in the mail() function, but only if it's
 * configured and only if we're not in safe_mode
 */
function forcereturn()
{
    global $_CONF, $config;

    // first check if safe_mode is on; if it is, we cannot pass -f
    $safemode = (int) ini_get('safe_mode');
    if ($safemode == 0)
    {
        if ($_CONF['forcereturn'] == 1)
        {
            $addparam = "-f" . $config['memail'];
        } else {
            $addparam = NULL;
        }
    } else {
        $addparam = NULL;
    }
    return($addparam);
}

/**
 * Function to construct an array that contains bugs column name, table
 * name, table column name, table column index name
 *
 * @params $sort the column to sort by
 */
function transsort($sort)
{
    switch($sort)
    {
        case "prod":
            $ret = array('bcolumn' => 'product', 'tname' => 'products', 'tcolumn' => 'name', 'tindex' => 'pid');
            break;
        case "comp":
            $ret = array('bcolumn' => 'component', 'tname' => 'components', 'tcolumn' => 'name', 'tindex' => 'cid');
            break;
        case "pri":
            $ret = array('bcolumn' => 'priority', 'tname' => 'priority_d', 'tcolumn' => 'id', 'tindex' => 'id');
            break;
        case "own":
            $ret = array('bcolumn' => 'assigned', 'tname' => 'users', 'tcolumn' => 'email', 'tindex' => 'uid');
            break;
        case "stat":
            $ret = array('bcolumn' => 'status', 'tname' => 'status_d', 'tcolumn' => 'name', 'tindex' => 'id');
            break;
        case "res":
            $ret = array('bcolumn' => 'resolution', 'tname' => 'resolve_d', 'tcolumn' => 'name', 'tindex' => 'id');
            break;
        case "summ":
            $ret = array('bcolumn' => 'shortdesc', 'tname' => 'bugs', 'tcolumn' => 'shortdesc', 'tindex' => 'bid');
            break;
        default:
            $ret = array('bcolumn' => 'bid', 'tname' => 'bugs', 'tcolumn' => 'bid', 'tindex' => 'bid');
            break;
    }
    return $ret;
}

/**
 * Function to run a series of configuration tests to make sure that things
 * are configured on the system properly
 */
function do_configtest()
{
    global $_CONF, $conf;

    if (!is_dir($_CONF['shadowdir']) || !is_writable($_CONF['shadowdir']))
    {
        print("<p><b><font color=red>" . gettext("shadowdir is not writable; please notify the administrator.") . "</font></b></p>");
    }
    if (!is_dir($_CONF['tmpdir']) || !is_writable($_CONF['tmpdir']))
    {
        print("<p><b><font color=red>" . gettext("temporary directory is not writable; please notify the administrator.") . "</font></b></p>");
    }
    $localgpgdir = $_CONF['root'] . $_CONF['gpgdir'];
    if (!is_dir($localgpgdir) || !is_writable($localgpgdir))
    {
        print("<p><b><font color=red>" . gettext("gnupg storage directory is not writable; please notify the administrator.") . "</font></b></p>");
    }
    $localattdir = $_CONF['root'] . $_CONF['attachdir'];
    if (!is_dir($localattdir) || !is_writable($localattdir))
    {
        print("<p><b><font color=red>" . gettext("attachments directory is not writable; please notify the administrator.") . "</font></b></p>");
    }
    if (ini_get("safe_mode") == 1)
    {
        // we are in safe_mode
        if (ini_get("safe_mode_exec_dir") == "")
        {
            print("<p><b><font color=red>" . gettext("safe_mode enabled, but no safe_mode_exec_dir defined - diff will be unable to run; please notify the administrator.") . "</font></b></p>");
        } else { // let's make sure that our difftool is in the exec_dir
            if (!ereg(ini_get("safe_mode_exec_dir"),$_CONF['difftool']))
            {
                print("<p><b><font color=red>" . gettext("diff is not in safe_mode_exec_dir and will not be able to run; please notify the administrator.") . "</font></b></p>");
            }
        }
    }
    // version test
    if ($conf['version'] != "")
    {
        $dbver = $config['version'];
    } else {
        $dbver = NULL;
    }
    if (isset($dbver) && $dbver != _VERSION)
    {
        print("<p><b><font color=red>" . sprintf(gettext("version mismatch: database version is %s while installed version is %s; please notify the administrator."), $dbver, _VERSION) . "</font></b></p>");
    }
    // upgrade test
    if (file_exists($_CONF['root'] . "/upgrade.php"))
    {
        print("<p><b><font color=red>" . gettext("WARNING: upgrade script is present!  Please notify the administrator.") . "</font></b></p>");
    }   
    // check if php version is 4.3.0 or less
    $pver = intval(str_replace(".", "", phpversion()));
    if ($pver < "430")
    {
        print("<p><b><font color=red>" . gettext(sprintf("minimum PHP version required is 4.3.0 and you have PHP version %s installed!", phpversion())) . "</font></b></p>");
    }
}

/**
 * Function to start every page 
 *
 * @params $ptitle the page title to print
 */
function d_StartPage($ptitle)
{
    global $_CONF, $rtmpldir, $wtmpldir, $config;

    // force adding charset in PHP headers
    header("Content-Type: text/html; charset=" . $_CONF['meta_lang']);
    $p_title = $config['sitename'];
    if (isset($ptitle))
    {
        $p_title .= " " . _NAME . ": " . $ptitle;
    }
    if (file_exists($rtmpldir . "/" . $config['theme'] . "/style.css"))
    {
        $_CONF['cssfile'] = $wtmpldir . "/" . $config['theme'] . "/style.css";
    }
    $data = <<<EOT
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
    <html>
    <head>
      <title>{$p_title}</title>
      <meta http-equiv="Content-Type" content="text/html; charset={$_CONF['meta_lang']}">
      <meta http-equiv="Pragma" content="no-cache">
      <link rel="stylesheet" href="{$_CONF['cssfile']}">
    </head>
    <body bgcolor="#ffffff" text="#000000" link="#00cccc" vlink="#00cccc">
EOT;

    print($data);
    do_configtest();
}

function d_DrawHeader($option)
{
    t_header($option);
}

function d_DrawNav($ulogin)
{
    global $_CONF, $_USER, $config;

    $username = NULL; $reg    = NULL; $login  = NULL; $pref = NULL; 
    $logout   = NULL; $submit = NULL; $newbug = NULL; $adm  = NULL;

    $home  = "index.php";
    $query = " | <a class=\"navbar\" href=\"query.php\">" . gettext("query bugs") . "</a>";

    if (!isset($ulogin))
    {
        $username = "none";
        $reg      = " | <a class=\"navbar\" href=\"newuser.php\">" . gettext("register") . "</a>";
        $login    = " | <a class=\"navbar\" href=\"login.php\">" . gettext("login") . "</a>";
    } else {
        $username = $ulogin;
        $pref     = " | <a class=\"navbar\" href=\"pref.php\">" . gettext("preferences") . "</a>";
        $logout   = " | <a class=\"navbar\" href=\"logout.php\">" . gettext("logout") . "</a>";
        $submit   = " | <a class=\"navbar\" href=\"enterbug.php\">" . gettext("submit a bug") . "</a>";
        $newbug   = " | <a class=\"navbar\" href=\"buglist.php?type=mine&myclass=new\">" . gettext("your open bugs") . "</a>"; 
    }
    if ($_USER['access'] >= $config['adminac'])
    {
        $adm      = " | <a class=\"navbar\" href=\"admin.php\">" . gettext("administration") . "</a>";
    }

    t_nav($home, $query, $submit, $reg, $newbug, $pref, $login, $logout, $username, $adm);
}

function d_DrawFooter()
{
    global $_CONF, $db;
    $name         = _NAME;
    $version      = _VERSION;
    $copyright    = gettext("Copyright &copy; 2001-2004 <a href=\"http://www.danen.ca/\">Danen Consulting Services</a>; all rights reserved.");
    $weburl       = $_CONF['webroot'];
    t_footer($name, $version, $copyright, $weburl);
    $db->disconnect();
    if ($_CONF['buffer'] == 1)
    {
        ob_end_flush();
    }
}

function d_DrawNewUser($pass = "0")
{
    global $config;
    $sitename     = $config['sitename'];
    $content      = stripslashes($config['newublurb']);
    $name         = _NAME;
    $form         = "newuser.php";
    if ($pass == "0")
    {
        t_newuser($sitename, $content, $name, $form);
    }
    t_forgotpass($name, $form);
}

function d_DrawQueryForm($uid)
{
    global $config, $db;

    $sitename = $config['sitename'];
    // get status
    $statopts = NULL; $useropts = NULL; $user2opts = NULL; $prodopts = NULL;
    $count    = 0;

    $result   = $db->query("SELECT * FROM status_d");
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $status[$count] = array("id" => $row['id'], "name" => $row['name']);
        $count++;
    }
    $result->free();

    while (list($key, $val) = each($status))
    {
        $statopts .= sprintf("<option value=\"%s\">%s</option>", $val['id'],$val['name']);
    }
    // get products
    $prod   = array();
    $count  = 0;
    $result = $db->query("SELECT * FROM products WHERE deleted = '0'");
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $prod[$count] = array("id" => $row['pid'], "name" => $row['name']);
        $count++;
    }
    $result->free();

    while (list($key, $val) = each($prod))
    {
        $prodopts .= sprintf("<option value=\"%s\">%s</option>", $val['id'],$val['name']);
    }
    // get users
    $count  = 0;
    $result = $db->query("SELECT * FROM users WHERE deleted = '0' ORDER BY email ASC");
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $user[$count] = array("uid" => $row['uid'], "email" => $row['email']);
        $count++;
    }
    $result->free();

    while (list($key, $val) = each($user))
    {
        $useropts .= sprintf("<option value=\"%s\">%s</option>", $val['uid'],do_transemail($val['email']));
    }
    // get users without current user
    $count  = 0;
    $user2  = NULL;
    $result = $db->query("SELECT * FROM users WHERE (uid != '$uid') AND (deleted = '0') ORDER BY email ASC");
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $user2[$count] = array("uid" => $row['uid'], "email" => $row['email']);
        $count++;
    }
    $result->free();

    if (is_array($user2))
    {
        while (list($key, $val) = each($user2))
        {
            $user2opts .= sprintf("<option value=\"%s\">%s</option>", $val['uid'],do_transemail($val['email']));
        }
    } else {
        $user2opts = "";
    }
    t_queryform($sitename, $statopts, $useropts, $user2opts, $prodopts);
}

function d_DrawPref($uid)
{
    global $_CONF, $config, $db;

    $result = $db->query("SELECT * FROM users WHERE uid = '$uid'");
    dberror($result);
    $user   = $result->fetchRow(DB_FETCHMODE_ASSOC);

    $sitename  = $config['sitename'];
    $content   = stripslashes($config['prefblurb']);
    $form      = "pref.php";
    $uname     = $user['uname'];
    $rname     = $user['name'];
    $email     = $user['email'];
    $cookieval = $user['cookietimeout'];
    $laston    = d_getTime($user['laston']);
    $thison    = d_getTime($user['thison']);
    $epref     = $user['epref'];
    if ($laston == "")
    {
        $laston = gettext("This is your first login");
    }
    // get gpg info
    if (isset($user['gpg']))
    {
        $fp        = fopen($_CONF['root'] . $user['gpg'], "r");
        $gpg       = fread($fp, filesize($_CONF['root'] . $user['gpg']));
        fclose($fp);
        $dkeyform  = "<form method=\"post\" action=\"" . $form . "\">\n";
        $dkeyform .= "<input type=\"hidden\" name=\"go\" value=\"deletekey\">\n";
        $dkeyform .= "<input type=\"submit\" value=\"" . gettext("Delete Key") . "\"></form>\n";
    } else {
        $gpg       = "";
        $dkeyform  = "";
    }

    t_pref($sitename, $content, $form, $uname, $rname, $email, $laston, $thison, $uid, $gpg, $dkeyform, $cookieval, $epref);
    $result->free();
}

function d_DrawLogin($caller)
{
    $form = "login.php";
    t_login($form, $caller);
}

function d_DrawEditProd($what, $pid = "0", $name = NULL, $desc = NULL)
{
    global $db;

    // we can have two possible values of $what: insert or change
    if ($what == "insert")
    {
        $button = gettext("Add");
    } elseif ($what == "change") {
        $button = gettext("Change");
    } else {
        print("invalid call to d_DrawEditProd()");
        exit;
    }
    if ($pid != "0")
    {
        $result  = $db->query("SELECT * FROM products WHERE pid = '$pid'");
        dberror($result);
        $product = $result->fetchRow(DB_FETCHMODE_ASSOC);
        if ($product['name'] != "")
        {
            $name    = $product['name'];
        }
        if ($product['description'] != "")
        {
            $desc    = $product['description'];
        }
        $result->free();
    }
    $p_string['product'] = gettext("Product");
    $p_string['desc']    = gettext("Description");

    $data = <<<EOT
        <form method="post" action="editproducts.php">
        <table border="0" cellpadding="0" cellspacing="0" width="100%">
            <tr>
                <td width="50%" align="right">{$p_string['product']}:</td>
                <td width="50%"><input type="text" name="name" size="50" maxlength="64" value="{$name}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['desc']}:</td>
                <td width="50%"><textarea name="description" cols="50" rows="2" wrap="Virtual">{$desc}</textarea></td>
            </tr>
            <tr>
                <td width="50%" align="right">
                    <input type="submit" value="{$button}">
                    <input type="hidden" name="action" value="{$what}">
                    <input type="hidden" name="pid" value="{$pid}">
                </td>
                <td width="50%">&nbsp;</td>
            </tr>
        </table>
        </form>
EOT;

    print($data);
}

function d_DrawEditVer($what, $vid, $pid = "0", $name = NULL)
{
    global $db;

    // we can have two possible values of $what: insert or change
    if ($what == "insert")
    {
        $button = gettext("Add");
    } elseif ($what == "change") {
        $button = gettext("Change");
    } else {
        print("invalid call to d_DrawEditVer()");
        exit;
    }
    if ($pid != "0")
    {
        $result  = $db->query("SELECT * FROM versions WHERE vid = '$vid'");
        dberror($result);
        $version = $result->fetchRow(DB_FETCHMODE_ASSOC);
        if ($version['name'] != "")
        {
            $name    = $version['name'];
        }
        $result->free();
    }

    $p_string['version'] = gettext("Version");

    $data = <<<EOT
        <form method="post" action="editversions.php">
        <table border="0" cellpadding="0" cellspacing="0" width="100%">
            <tr>
                <td width="50%" align="right">{$p_string['version']}:</td>
                <td width="50%"><input type="text" name="name" size="50" maxlength="64" value="{$name}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">
                    <input type="submit" value="{$button}">
                    <input type="hidden" name="action" value="{$what}">
                    <input type="hidden" name="vid" value="{$vid}">
                    <input type="hidden" name="pid" value="{$pid}">
                </td>
                <td width="50%">&nbsp;</td>
            </tr>
        </table>
        </form>
EOT;

    print($data);
    $result->free();
}

function d_DrawEditComp($what, $cid, $pid)
{
    global $db, $config;

    // we can have two possible values of $what: insert or change
    if ($what == "insert")
    {
        $button = gettext("Add");
        $uid['uid']               = NULL;
        $component['name']        = NULL;
        $component['description'] = NULL;
        $component['autocc']      = NULL;
    } elseif ($what == "change") {
        $button = gettext("Change");
        $result    = $db->query("SELECT * FROM components WHERE cid = '$cid'");
        dberror($result);
        $component = $result->fetchRow(DB_FETCHMODE_ASSOC);
        $owner     = $component['owner'];
        $result->free();
        $result    = $db->query("SELECT * FROM users WHERE uid = '$owner'");
        dberror($result);
        $uid       = $result->fetchRow(DB_FETCHMODE_ASSOC);
        $result->free();
    } else {
        print("invalid call to d_DrawEditComp()");
        exit;
    }

    $p_string['component'] = gettext("Component");
    $p_string['desc']      = gettext("Description");
    $p_string['inown']     = gettext("Initial Owner");
    $p_string['autocc']    = gettext("Auto CC (separate by commas)");

    $result = $db->query("SELECT * FROM users WHERE (access >= '" . $config['devac'] . "') AND (deleted = '0')");
    dberror($result);

    $selectlist = NULL;
    while ($user = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $sel  = "";
        if ($uid['uid'] == $user['uid'])
        {
            $sel = "selected";
        }
        $selectlist .= sprintf("<option value=\"%s\" %s>%s</option>\n",$user['uid'],$sel,$user['email']);
    }
    $result->free();

    $data = <<<EOT
        <form method="post" action="editcomponents.php">
        <table border="0" cellpadding="0" cellspacing="0" width="100%">
            <tr>
                <td width="50%" align="right">{$p_string['component']}:</td>
                <td width="50%"><input type="text" name="name" size="50" maxlength="64" value="{$component['name']}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['desc']}:</td>
                <td width="50%"><textarea name="description" cols="50" rows="2" wrap="Virtual">{$component['description']}</textarea></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['inown']}:</td>
                <td width="50%">
                    <select name="owner" value="{$uid['uid']}">
                    {$selectlist}
                    </select>         
                </td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['autocc']}:</td>
                <td width="50%"><input type="text" name="autocc" size="50" maxlength="100" value="{$component['autocc']}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">
                    <input type="submit" value="{$button}">
                    <input type="hidden" name="action" value="{$what}">
                    <input type="hidden" name="cid" value="{$cid}">
                    <input type="hidden" name="pid" value="{$pid}">
                </td>
                <td width="50%">&nbsp;</td>
            </tr>
        </table>
        </form>
EOT;

    print($data);
    print("<p>" . gettext("Auto CC is used to automatically add email addresses to new bugs assigned to this component.  This is a comma delimited list (ie. 'joe@user.com,jim@other.com')") . "</p>");
}

function d_DrawEditUser($moduserid)
{
    global $config, $db;

    $result = $db->query("SELECT * FROM users WHERE uid = '$moduserid'");
    dberror($result);
    $utmp   = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $result->free();

    $p_string['username'] = gettext("User Name");
    $p_string['realname'] = gettext("Real Name");
    $p_string['emailadd'] = gettext("Email Address");
    $p_string['acslvl']   = gettext("Access Level");
    $p_string['newpass']  = gettext("New Password");
    $p_string['newpass2'] = gettext("New Password (re-enter)");
    $p_string['change']   = gettext("Change");

    // check if user is a basic user
    if ($utmp['access'] == $config['defaultac'])
    {
       $optbasic = sprintf("<option value=\"%s\" SELECTED>%s</option>", $config['defaultac'], gettext("Basic User"));
    } else {
       $optbasic = sprintf("<option value=\"%s\">%s</option>", $config['defaultac'], gettext("Basic User"));
    }
    // check if user is a developer
    if ($utmp['access'] == $config['devac'])
    {
       $optdevel = sprintf("<option value=\"%s\" SELECTED>%s</option>", $config['devac'], gettext("Developer"));
    } else {
       $optdevel = sprintf("<option value=\"%s\">%s</option>", $config['devac'], gettext("Developer"));
    }
    // check if user is an admin
    if ($utmp['access'] == $config['adminac'])
    {
       $optadmin = sprintf("<option value=\"%s\" SELECTED>%s</option>", $config['adminac'], gettext("Administrator"));
    } else {
       $optadmin = sprintf("<option value=\"%s\">%s</option>", $config['adminac'], gettext("Administrator"));
    }

    $data = <<<EOT
        <form method="post" action="editusers.php">
        <table border="0" cellpadding="0" cellspacing="0" width="100%">
            <tr>
                <td width="50%" align="right">{$p_string['username']}:</td>
                <td width="50%"><input type="text" name="uname" size="50" maxlength="64" value="{$utmp['uname']}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['realname']}:</td>
                <td width="50%"><input type="text" name="name" size="50" maxlength="64" value="{$utmp['name']}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['emailadd']}:</td>
                <td width="50%"><input type="text" name="email" size="50" maxlength="64" value="{$utmp['email']}"></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['acslvl']}:</td>
                <td width="50%">
                    <select name ="access">
                    {$optbasic}
                    {$optdevel}
                    {$optadmin}
                    </select>
                </td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['newpass']}:</td>
                <td width="50%"><input type="password" name="pass1" size="50" maxlength="64" value=""></td>
            </tr>
            <tr>
                <td width="50%" align="right">{$p_string['newpass2']}</td>
                <td width="50%"><input type="password" name="pass2" size="50" maxlength="64" value=""></td>
            </tr>
            <tr>
                <td width="50%" align="right">
                    <input type="submit" value="{$p_string['change']}">
                    <input type="hidden" name="action" value="change">
                    <input type="hidden" name="moduserid" value="{$moduserid}">
                </td>
                <td width="50%">&nbsp;</td>
            </tr>
        </table>
        </form>
EOT;

    print($data);
}

function d_DrawCAttach($bug)
{
    global $config;

    t_cattach($bug, $config['maxfsize']);
}

function d_DoDiff($bugnum)
{
    global $_CONF, $config, $db, $_USER;

    $result   = $db->query("SELECT * FROM bugs WHERE bid = '$bugnum'");
    dberror($result);
    $bug      = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $result->free();
    $assigned = $db->getOne("SELECT email FROM users WHERE uid = '" . $bug['assigned'] . "'");
    $result   = $db->query("SELECT epref,email FROM users WHERE uid = '" . $_USER['uid'] . "'");
    dberror($result);
    $row      = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $callinguser = NULL;
    if ($row['epref'] == 0)
    {
        // user doesn't want to receive their changes
        $callinguser = $row['email'];
    }
    $result->free();
    $reported = $db->getOne("SELECT email FROM users WHERE uid = '" . $bug['reporter'] . "'");
    $statx    = $db->getOne("SELECT name FROM status_d WHERE id = '" . $bug['status'] . "'");
    if ($bug['resolution'] != "")
    {
        $resolve = $db->getOne("SELECT name FROM resolve_d WHERE id = '" . $bug['resolution'] . "'");
        if ($bug['fixedver'] != "")
        {
            $fixver = $bug['fixedver'];
        } else {
            $fixver = NULL;
        }
    } else {
        $resolve = NULL;
    }
    $priority = $db->getOne("SELECT name FROM priority_d WHERE id = '" . $bug['priority'] . "'");
    $compt    = $db->getOne("SELECT name FROM components WHERE cid = '" . $bug['component'] . "'");
    $product  = $db->getOne("SELECT name FROM products WHERE pid = '" . $bug['product'] . "'");
    $version  = $db->getOne("SELECT name FROM versions WHERE vid = '" . $bug['version'] . "'");

    if ($bug['private'] == 0)
    {
        $priv   = "PUBLIC";
    } else {
        $priv   = "PRIVATE";
    }

    // get the maximum string length to format the email nicely
    // we do not use gettext here; all diffs are abitrarily in english
    // (this prevents multi-language erroneous diffs if users use
    // different languages)
    $msg[0]   = "Bug#";
    $msg[1]   = "Product";
    $msg[2]   = "Version";
    $msg[3]   = "Status";
    $msg[4]   = "Priority";
    $msg[5]   = "Component";
    $msg[6]   = "URL";
    $msg[7]   = "Assigned To";
    $msg[8]   = "Reported By";
    $msg[9]   = "Type";
    $msg[10]  = "Summary";
    $msg[11]  = "Resolution";
    $msg[12]  = "Fixed Ver";
    $limit    = count($msg);
    $size     = 0;
    for ($i=0; $i < $limit; $i++)
    {
        if (strlen($msg[$i]) > $size)
        {
            $size = strlen($msg[$i]);
        }
    }
    for ($i=0; $i < $limit; $i++)
    {
        $buff = $size - strlen($msg[$i]);
        if ($buff > 0)
        {
            $msg[$i] = $msg[$i] . str_repeat(" ", $buff);
        }
    }

    // need to find way to format a little nicer dependant on length of
    // defined strings
    $tmpdesc    = $msg[0]  . ": " . stripslashes($bugnum) . "\n";
    $tmpdesc   .= $msg[1]  . ": " . stripslashes($product) . "\n";
    $tmpdesc   .= $msg[2]  . ": " . stripslashes($version) . "\n";
    $tmpdesc   .= $msg[3]  . ": " . $statx . "\n";
    if (isset($resolve))
    {
        $tmpdesc .= $msg[11] . ": " . $resolve . "\n";
        $tmpdesc .= $msg[12] . ": " . $fixver . "\n";
    }
    $tmpdesc   .= $msg[4]  . ": " . $priority . "\n";
    $tmpdesc   .= $msg[5]  . ": " . stripslashes($compt) . "\n";
    $tmpdesc   .= $msg[6]  . ": " . stripslashes($bug['url']) . "\n";
    $tmpdesc   .= $msg[7]  . ": " . $assigned . "\n";
    $tmpdesc   .= $msg[8]  . ": " . $reported . "\n";
    $tmpdesc   .= $msg[9]  . ": " . $priv . "\n\n";
    $tmpdesc   .= $msg[10] . ": " . stripslashes($bug['shortdesc']) . "\n\n";

    $result = $db->query("SELECT * FROM bugdesc WHERE bid = '" . $bug['bid'] . "' ORDER BY did");
    dberror($result);
    while ($ldesc = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $result2 = $db->query("SELECT * FROM users WHERE uid = '" . $ldesc['b_user'] . "'");
        dberror($result2);
        $rep     = $result2->fetchRow(DB_FETCHMODE_ASSOC);
        $tmpdesc .= sprintf("Comment posted: %s by %s", d_getTime($ldesc['b_date']), $rep['email']) . "\n\n";
        $tmpdesc .= stripslashes($ldesc['description']) . "\n\n";
        $result2->free();
    }
    $result->free();

    $oldfile = $_CONF['shadowdir'] . "/" . $bugnum;
    if ($tmpfile = tempnam($_CONF['shadowdir'],$bugnum . ".tmp."))
    {
        $fp = fopen($tmpfile, "w");
        fwrite($fp, $tmpdesc);
        fclose($fp);
    } else {
        print("<p><font color=red>" . gettext("Error: Cannot open temporary file.") . "</font></p>");
    }

    // we need an $oldfile or we don't get a diff
    touch($oldfile);

    // create the diff
    $stmpfile = ereg_replace($_CONF['shadowdir'] . "/","",strtr(addcslashes($tmpfile, "\0..\37!@\177..\377"), "\\", "/"));
    $cmd      = sprintf("%s -auN %s/$bugnum %s/$stmpfile",$_CONF['difftool'],$_CONF['shadowdir'],$_CONF['shadowdir']);
    $output   = exec($cmd,$diffarray,$return);
    if ($return == 1)
    {
        // there are differences
        $diffmsg = implode($diffarray, "\n");
        $diffmsg = ereg_replace($_CONF['shadowdir'] . "/", "", $diffmsg);
        $diffmsg = str_replace(" query.php", " " . $_CONF['webroot'] . "/query.php", $diffmsg);
        $diffmsg = str_replace("showattachment.php", $_CONF['webroot'] . "/showattachment.php", $diffmsg);
    } else {
        $diffmsg = "";
    }

    // move new bug desc file to old
    unlink($oldfile) or die ("Cannot delete $oldfile");
    rename($tmpfile, $oldfile) or die ("Cannot rename $tmpfile to $oldfile");

    // send email to assigned user and all cc's
    $message = $_CONF['webroot'] . "/query.php?bug=" . $bugnum . "\n\n";
    $result = $db->query("SELECT * FROM bugs WHERE bid = '$bugnum'");
    dberror($result);
    $bugx   = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $result->free();

    $diffnum  = count($diffmsg);
    if ($diffnum == 0)
    {
        // print("<span class=\"text\"><font color=\"red\">" . gettext("There was an error or you entered no information when saving changes; diff is empty!") . "</font></span>");
        $message .= "A diff was not generated; please access this bug to see what changes were made.\n\n";
    }

    $message .= $diffmsg;
    $message .= "\n\n";
    $message .= str_replace("<br>", "\n", stripslashes($config['emailfooter']));

    $result = $db->query("SELECT * FROM users WHERE uid = '" . $bugx['assigned'] . "'");
    dberror($result);
    $userx  = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $result->free();

    // make our cc list, try and remove all duplicate addresses 
    $cclist = $userx['email'] . ", " . $reported;
    if ($bugx['cc'] != "")
    {
        // make sure we don't add an extra ", " to the list this way
        $cclist = $cclist . ", " . $bugx['cc'];
    }
    $cclist = explode(", ",$cclist);
    $tempcc = array_unique($cclist);
    $cc     = array();
    foreach ($tempcc as $element)
    {
        if ($element != $callinguser)
        {
            // user doesn't want mail
            array_push($cc, $element);
        }
    }
 
    // reconstruct the list
    if (count($cc) > 1)
    {
        // more than one recipient
        $emailto = $cc[0];
        unset($cc[0]);
        $cc = implode($cc, ", ");
    } else {
        $emailto = $cc[0];
        $cc = "";
    }
    
    $headers    = "Date: " . date("r") . $_CONF['crlf'];
    $headers   .= "From: " . $config['mdaemon'] . " <" . $config['memail'] . ">" . $_CONF['crlf'];
    $headers   .= "CC: " . $cc . $_CONF['crlf'];
    if ($config['mdaemonreplyto'] != "")
    {
        $headers .= "Reply-To: " . $config['mdaemonreplyto'] . $_CONF['crlf'];
    }
    $headers   .= "Return-Path: <" . $config['memail'] . ">" . $_CONF['crlf'];
    $headers   .= "Content-Type: text/plain; charset=" . $_CONF['meta_lang'] . $_CONF['crlf'];
    $headers   .= "X-Mailer: " . _NAME . " " . _VERSION . $_CONF['crlf'];
    $headers   .= "X-Anthill-Product: " . stripslashes($product) . $_CONF['crlf'];
    $headers   .= "X-Anthill-Component: " . stripslashes($compt) . $_CONF['crlf'];
    $headers   .= "X-Anthill-Priority: $priority" . $_CONF['crlf'];
    $addparam   = forcereturn();

    if ($emailto != "")
    {
        if ($addparam != "")
        {
            $retval = @mail($emailto,
                      "[Bug #" . $bugnum . "] [" . stripslashes($compt) . "] " . stripslashes($bug['shortdesc']), $message,
                      $headers, $addparam);
        } else {
            $retval = @mail($emailto,
                      "[Bug #" . $bugnum . "] [" . stripslashes($compt) . "] " . stripslashes($bug['shortdesc']), $message,
                      $headers);
        }
    }
}

function d_DoSearchList($search, $sort, $order, $pidnum = 2)
{
    global $user, $db;

    $sortorder = transsort($sort);

    $c      = 0;
    $bcol   = "#ffffff";
    // query retrieves only the bugid (bid)
    $count  = 0;
    if ($sortorder['tname'] == "bugs")
    {
        $j     = "";
        $join  = "";
        $sord  = " $order";
    } else {
        $j     = "(";
        $join  = "LEFT JOIN " . $sortorder['tname'] . " ON bugs." . $sortorder['bcolumn'];
        $join .= " = " . $sortorder['tname'] . "." . $sortorder['tindex'] . ") ";
        $sord  = " $order,bugs.bid ASC";
    }
    
    if ($sortorder['tname'] == "bugs")
    {
        $j     = "";
        $join  = "";
        $sord  = " $order";
    } else {
        $j     = "(";
        $join  = "LEFT JOIN " . $sortorder['tname'] . " ON bugs." . $sortorder['bcolumn'];
        $join .= " = " . $sortorder['tname'] . "." . $sortorder['tindex'] . ") ";
        $sord  = " $order,bugs.bid ASC";
    }
    
    $query  = "SELECT DISTINCT(bugs.bid) AS bugid FROM " . $j . "(bugs LEFT JOIN bugdesc ON bugs.bid = bugdesc.bid) ";
    $query .= $join;
    $query .= "WHERE (bugs.shortdesc like '%$search%' ";
    $query .= "OR bugs.shortdesc like '$search%' ";
    $query .= "OR bugs.shortdesc like '%$search' ";
    $query .= "OR bugdesc.description like '%$search%' ";
    $query .= "OR bugdesc.description like '$search%' ";
    $query .= "OR bugdesc.description like '%$search') ";
    $query .= "ORDER BY " . $sortorder['tname'] . "." . $sortorder['tcolumn'] . $sord;

    $result = $db->query($query);
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $btmp[$count] = $row['bugid'];
        $count++;
    }
    $result->free();

    $lastbid = 0;
    $x       = 0;
    $data    = NULL;
    for ($i=0; $i<count($btmp); $i++)
    {
        if ($btmp[$i] != $lastbid)
        {
            $result = $db->query("SELECT * FROM bugs WHERE bid = '" . $btmp[$i] . "'");
            dberror($result);
            $bugs   = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if ($bugs['private'] == 1 && !isset($user['uid']))
            {
                continue;
            }
            $result->free();
            $x++;
            // set background row color (make a pref later?)
            if ($c == 0)
            {
                $bcol = "#ffffff";
                $c    = 1;
            } else {
                $bcol = "#e7e7e7";
                $c    = 0;
            }

            $p_compname = $db->getOne("SELECT name FROM products WHERE pid = '" . $bugs['product'] . "'");
            $p_prodname = $db->getOne("SELECT name FROM components WHERE cid = '" . $bugs['component'] . "'");
            $p_priname  = $db->getOne("SELECT name FROM priority_d WHERE id = '" . $bugs['priority'] . "'");
            $p_emailass = $db->getOne("SELECT name FROM users WHERE uid = '" . $bugs['assigned'] . "'");
            $p_statname = $db->getOne("SELECT name FROM status_d WHERE id = '" . $bugs['status'] . "'");
            if ($bugs['resolution'] != "" && $bugs['resolution'] != "4") {
                $p_resname  = $db->getOne("SELECT name FROM resolve_d WHERE id = '" . $bugs['resolution'] . "'");
            } else {
                $p_resname  = NULL;
            }
            $p_shortdesc = stripslashes($bugs['shortdesc']);

            $data .= <<<EOT
                <tr bgcolor="{$bcol}">
                    <td valign="top"><a href="query.php?bug={$bugs['bid']}">{$bugs['bid']}</a>&nbsp;</td>
EOT;
            if ($pidnum > 1)
            {
                $data .= <<<EOT
                    <td valign="top">{$p_prodname}&nbsp;</td>
EOT;
            }
            $data .= <<<EOT
                    <td valign="top">{$p_compname}&nbsp;</td>
                    <td valign="top">{$p_priname}&nbsp;</td>
                    <td valign="top">{$p_emailass}&nbsp;</td>
                    <td valign="top">{$p_statname}&nbsp;</td>
                    <td valign="top">{$p_resname}&nbsp;</td>
                    <td valign="top">{$p_shortdesc}</td>
                </tr>
EOT;
        }
        $lastbid = $btmp[$i];
    }
    print($data);
    return($x);
}

function d_DoBugList($where, $sort, $order, $pidnum = 2)
{
    global $user, $_CONF, $db;

    $sortorder = transsort($sort);

    $c      = 0;
    $bcol   = "#ffffff";
    
    if ($sortorder['tname'] == "bugs")
    {
        $j     = "";
        $join  = "";
        $sord  = " $order";
    } else {
        $j     = "(";
        $join  = "LEFT JOIN " . $sortorder['tname'] . " ON bugs." . $sortorder['bcolumn'];
        $join .= " = " . $sortorder['tname'] . "." . $sortorder['tindex'] . ") ";
        $sord  = " $order,bugs.bid ASC";
    }
    $query  = "SELECT * FROM ". $j . "bugs " . $join . " " . $where;
    $query .= "ORDER BY " . $sortorder['tname'] . "." . $sortorder['tcolumn'] . $sord;   
    $result = $db->query($query);
    dberror($result);

    $count  = 0;
    $data   = NULL;
    while ($bugs = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        if ($bugs['private'] == 1 && !isset($user['uid']))
        {
            continue;
        }
        $count++;
        // set background row color (make a pref later?)
        if ($c == 0)
        {
            $bcol = "#ffffff";
            $c    = 1;
        } else {
            $bcol = "#e7e7e7";
            $c    = 0;
        }

        $p_prodname = $db->getOne("SELECT name FROM products WHERE pid = '" . $bugs['product'] . "'");
        $p_compname = $db->getOne("SELECT name FROM components WHERE cid = '" . $bugs['component'] . "'");
        $p_priname  = $db->getOne("SELECT name FROM priority_d WHERE id = '" . $bugs['priority'] . "'");
        $p_emailass = $db->getOne("SELECT name FROM users WHERE uid = '" . $bugs['assigned'] . "'");
        $p_statname = $db->getOne("SELECT name FROM status_d WHERE id = '" . $bugs['status'] . "'");
        if ($bugs['resolution'] != "" && $bugs['resolution'] != "4")
        {
            $p_resname = $db->getOne("SELECT name FROM resolve_d WHERE id = '" . $bugs['resolution'] . "'");
        } else {
            $p_resname = NULL;
        }
        $p_shortdesc = stripslashes($bugs['shortdesc']);

        $data .= <<<EOT
            <tr bgcolor="{$bcol}">
                <td valign="top"><a href="query.php?bug={$bugs['bid']}">{$bugs['bid']}</a>&nbsp;</td>
EOT;
        if ($pidnum > 1)
        {
            $data .= <<<EOT
                <td valign="top">{$p_prodname}&nbsp;</td>
EOT;
        }
        $data .= <<<EOT
                <td valign="top">{$p_compname}&nbsp;</td>
                <td valign="top">{$p_priname}&nbsp;</td>
                <td valign="top">{$p_emailass}&nbsp;</td>
                <td valign="top">{$p_statname}&nbsp;</td>
                <td valign="top">{$p_resname}&nbsp;</td>
                <td valign="top">{$p_shortdesc}</td>
            </tr>
EOT;
    }
    print($data);
    return($count);
}

function d_mailUser($subject)
{

    global $config, $db, $_CONF;

    $bugurl = $_CONF['webroot'] . "/query.php?bug=BUG";

    switch($subject)
    {
        case "NEW":
            $msg   = sprintf("There are new bugs assigned to you.  Please accept them or assign them to\nsomeone else by going to %s\n\n", $bugurl);
            $query = "SELECT assigned, shortdesc, bid FROM bugs WHERE status = '1'";
            break;
        case "UNRESOLVED":
            $msg   = sprintf("There are unresolved bugs assigned to you. This is a reminder that you\nhave open bugs in the bug system.\n\n%s\n\n", $bugurl);
            $query = "SELECT assigned, shortdesc, bid FROM bugs WHERE (status = '2' OR status = '3' OR status = '5')";
            break;
    }

    $result = $db->query($query);
    dberror($result);
    if ($result->numRows() > 0)
    {
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
        {
            $bid       = $row['bid'];
            $shortdesc = stripslashes($row['shortdesc']);
            $assigned  = $row['assigned'];
            $email     = $db->getOne("SELECT email FROM users WHERE uid = '$assigned'");
            $result2   = $db->query("SELECT b_user, description FROM bugdesc WHERE bid = '$bid' ORDER BY date LIMIT 1");
            dberror($result2);
            $row2      = $result2->fetchRow(DB_FETCHMODE_ASSOC);
            $rid       = $row2['b_user'];
            $ldesc     = $row2['description'];
            $result2->free();
            $reporter  = $db->getOne("SELECT email FROM users WHERE uid = '$rid'");

            $message  = str_replace("BUG","$bid",$msg);
            $message .= "\n\nReported by: $reporter";
            $message .= "\n\nShort Description:\n$shortdesc";
            $message .= "\n\nLong Description:\n" . stripslashes($ldesc);
            $message .= "\n\n--\n" . $_CONF['webroot'];
        
            $headers  = "Date: " . date("r") . $_CONF['crlf'];
            $headers .= "From: " . $config['mdaemon'] . " <" . $config['memail'] . ">" . $_CONF['crlf'];
            $headers .= "Return-Path: <" . $config['memail'] . ">" . $_CONF['crlf'];
            $headers .= "X-Mailer: " . _NAME . " " . _VERSION . $_CONF['crlf'];
            $addparam = forcereturn();
        
            if ($addparam != "")
            {
                $retval = @mail($email, "[$subject BUG #$bid] $shortdesc", $message, $headers, $addparam);
            } else {
                $retval = @mail($email, "[$subject BUG #$bid] $shortdesc", $message, $headers);
            }
        }
    }
    $result->free();
}
  
function do_transurl($url, $blank)
{
    // if this is an escaped url (ie. someone entered it using <a href="">bla</a> do not htmlize it as
    // a) it would be bad form and b) it looks horrible.
    if (ereg(".*lt;a\ href.*quot;.*gt;.*",$url) || ereg(".*quot;.*gt;.*",$url)) return($url);
    if ($blank==1)
    {
        $trans = preg_replace("/((http(s?):\/\/)|(www\.))([\S\.]+)\b/i",
          "<a href=\"http$3://$4$5\" target=\"_blank\">$2$4$5</a>", $url);
    } else {
        $trans = preg_replace("/((http(s?):\/\/)|(www\.))([\S\.]+)\b/i",
          "<a href=\"http$3://$4$5\">$2$4$5</a>", $url);
    }
    return($trans);
}

/**
 * Function to obfuscate email addresses to keep spam harvesting down
 *
 * @param $email the email address to translate
 */
function do_transemail($email)
{
    $email = ereg_replace("@", "__at__", $email);
    return($email);
}

function d_doQuery($uid, $type)
{
    global $PHP_SELF, $_CONF;

    if (empty($uid))
    {
        $uid = 0;
    }
    if ($type == "a" && $_CONF['advq'] == 1)
    {
        print("<p>" . gettext("Advanced bug searching and reporting capabilities query.") . "</p>");
        z_ShowSelectionGrid();
        print("<p><a href=\"$PHP_SELF?z=s\">" . gettext("Simple bug search") . "</a></p>");
    } else {
        print("<p>" . gettext("Simple bug searching and reporting query.") . "</p>");
        d_DrawQueryForm($uid);
        if ($_CONF['advq'] == 1)
        {
            print("<p><a href=\"$PHP_SELF?z=a\">" . gettext("Advanced bug search") . "</a></p>");
        }
    }
}

function d_DumpVdiff($data)
{
    print(sprintf("<tr><td><a href=\"query.php?bug=%s\">%s</a></td>",$data['bid'],$data['bid']));
    print("<td>" . $data['fixedver'] . "</td>");
    print("<td>" . $data['shortdesc'] . "</td></tr>");
}

function d_AskVersion($product)
{
    global $db;

    $pname = $db->getOne("SELECT name FROM products WHERE pid = '$product'");

    print("<p>" . sprintf(gettext("Requested difference between versions  for %s..."), $pname) . "</p>");

    $p_string['checkdiff'] = gettext("Check differences between version ");
    $p_string['andver']    = " " . gettext("and version") . " ";
    $p_string['check']     = gettext("Check");

    $data = <<<EOT
        <form method="post" action="buglist.php">
        <input type="hidden" name="product" value="{$product}">
        <table cellpadding="0" cellspacing="0" border="0" width="100%">
            <tr>
                <td valign="center">
                    {$p_string['checkdiff']}
                    <input type="text" name="sversion" size="8" maxsize="8">  
                    {$p_string['andver']}
                    <input type="text" name="eversion" size="8" maxsize="8">  
                </td>
            </tr>
            <tr>
                <td><br /><input type="submit" value="{$p_string['check']}"></td>
            </tr>
        </table>
        </form>
EOT;

    print($data);
}

/**
 * Handles drawing the rules shown before a bug submission is allowed (if
 * so defined)
 */
function d_DrawRules()
{
    t_rules();
    t_rulesbutton();
}

/**
 * Handle database errors and debug logging
 *
 * @param $result the result identifier from a db->query
 */
function dberror($result) 
{
    global $_CONF, $PHP_SELF;

    if (DB::isError($result)) 
    {
        $string  = $result->getMessage();
        
        if ($_CONF['debug'] == 1) 
        {   // give the query as well
            // if debug_log is not set, debug goes to browser, otherwise it only goes to the logfile
            if ($_CONF['debug_log'] != "")
            {
                // write to our debug logfile
                $log    = date("M d H:m:s", mktime());
                $log   .= " [$PHP_SELF] $string; debug: " . $result->getDebugInfo() . "\n";
                $fp     = fopen($_CONF['debug_log'], "a+") or die("Could not open debug log");
                fwrite($fp, $log);
                fclose($fp);
                $string = "<p>" . gettext("<font color=\"red\">An error occured.</font>  Please inform the site administrator.") . "</p>";
            } else {
                $string .= "<br />Debug: " . $result->getDebugInfo();
            }
        }
        die ($string);
    }
}

/**
 * Function to build report rows for 'Open Bug Count By Owner'
 *
 * @param $user the uid of the user to report on
 * @param $name the name of the user to report on
 * @param $qtag as defined in reports
 */
function d_repassignrow($user, $name, $qtag)
{
    global $db;

    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '1' %s", $qtag));
    dberror($result);
    $return['new'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '5' %s", $qtag));
    dberror($result);
    $return['ver'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '2' %s", $qtag));
    dberror($result);
    $return['ass'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '0' %s", $qtag));
    dberror($result);
    $return['unc'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '3' %s", $qtag));
    dberror($result);
    $return['rop'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '7' %s", $qtag));
    dberror($result);
    $return['val'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '8' %s", $qtag));
    dberror($result);
    $return['nwo'] = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '$user' AND status = '9' %s", $qtag));
    dberror($result);
    $return['nwi'] = $result->numRows();
    $result->free();

    $return['tot'] = $return['new'] + $return['ver'] + $return['ass'] + $return['rop'] + $return['val'] + $return['nwo'] + $return['nwi'];
    $retdata       = NULL;

    if ($return['tot'] > 0)
    {
        // build the row
        $retdata = <<<EOT
            <tr>
                <td width="20%">{$name}</td>
                <td width="5%" align="center">{$return['new']}</td>
                <td width="10%" align="center">{$return['ass']}</td>
                <td width="10%" align="center">{$return['ver']}</td>
                <td width="10%" align="center">{$return['unc']}</td>
                <td width="10%" align="center">{$return['rop']}</td>
                <td width="10%" align="center">{$return['val']}</td>
                <td width="10%" align="center">{$return['nwo']}</td>
                <td width="10%" align="center">{$return['nwi']}</td>
                <td width="5%" align="center">{$return['tot']}</td>
            </tr>
EOT;
    }
    return($retdata);
}

/**
 * Function to build report rows for 'Bug Count by Reporter'
 *
 * @param $user the uid of the user to report on
 * @param $name the name of the user to report on
 * @param $qtag as defined in reports
 */
function d_replastreprow($user, $name, $qtag)
{
    global $db;

    $result      = $db->query(sprintf("SELECT bid,creation FROM bugs WHERE reporter = '%s' %s ORDER BY creation DESC", $user, $qtag));
    dberror($result);
    $rep         = $result->numRows();
    $row         = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $lastrepdate = d_getTime($row['creation']);
    $result->free();
    $retdata     = NULL;

    if ($rep > 0)
    {
        $retdata = <<<EOT
            <tr>
                <td width="20%">{$name}</td>
                <td width="10%" align="center">{$rep}</td>
                <td width="30%" colspan="3" align="center">{$lastrepdate}</td>
                <td colspan="2">&nbsp;</td>
            </tr>
EOT;

    }
    return($retdata);
}

/**
 * Function to build report rows for 'Summary of Participant Comments'
 *
 * @param $qtag3 as defined in reports
 */
function d_repsummcomm($qtag3)
{
    global $db;

    $retdata = NULL;
    $result  = $db->query("SELECT uid,name FROM users WHERE deleted = '0' ORDER BY name");
    dberror($result);
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        // this is my first-ever JOIN so be nice
        $result2 = $db->query(sprintf("SELECT bugdesc.b_date FROM bugdesc LEFT JOIN bugs ON bugdesc.bid=bugs.bid WHERE (bugdesc.b_user = '" . $row['uid'] . "') AND (bugdesc.system = '0') %s ORDER BY bugdesc.b_date DESC", $qtag3));
        dberror($result2);
        $rep         = $result2->numRows();
        $row2        = $result2->fetchRow(DB_FETCHMODE_ASSOC);
        $lastrepdate = d_getTime($row2['b_date']);
        $result2->free();

        if ($rep > 0)
        {
            $thirty    = date("YmdHis", mktime(0,0,0,date("m")-1,date("d"),date("Y")));
            $result2   = $db->query(sprintf("SELECT bugdesc.b_date FROM bugdesc LEFT JOIN bugs ON bugdesc.bid=bugs.bid WHERE (bugdesc.b_user = '" . $row['uid'] . "') AND (bugdesc.system = '0') AND (bugdesc.b_date > '$thirty') %s ORDER BY bugdesc.b_date DESC", $qtag3));
            dberror($result2);
            $last      = $result2->numRows();
            $result2->free();
            $retdata  .= <<<EOT
                <tr>
                    <td width="20%">{$row['name']}</td>
                    <td width="10%" align="center">{$last}</td>
                    <td width="10%" align="center">{$rep}</td>
                    <td width="30%" colspan="3" align="center">{$lastrepdate}</td>
                    <td>&nbsp;</td>
                </tr>
EOT;
        }
    }
    $result->free();
    return($retdata);
}

/**
 * Function to build report rows for 'Individual Open Bugs by Owner'
 *
 * @param $user the uid of the user to report on
 * @param $name the name of the user to report on
 * @param $qtag as defined in reports
 */
function d_repopenbugs($user, $name, $qtag)
{
    global $db;

    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '1' %s", $user, $qtag));
    dberror($result);
    $new     = $result->numRows();
    $newlist = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $newlist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$newlist, $row['bid'], $row['bid']);
    }
    $result->free();
    $result  = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '2' %s", $user, $qtag));
    dberror($result);
    $assigned = $result->numRows();
    $asslist  = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $asslist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$asslist, $row['bid'], $row['bid']);
    }
    $result->free();
    $result  = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '5' %s", $user, $qtag));
    dberror($result);
    $verified = $result->numRows();
    $verlist  = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $verlist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$verlist, $row['bid'], $row['bid']);
    }
    $result->free();
    $result     = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '0' %s", $user, $qtag));
    dberror($result);
    $unconfirmed = $result->numRows();
    $unconlist   = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $unconlist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$unconlist, $row['bid'], $row['bid']);
    }
    $result->free();
    $result  = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '3' %s", $user, $qtag));
    dberror($result);
    $reopened = $result->numRows();
    $reoplist = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $reoplist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$reoplist, $row['bid'], $row['bid']);
    }
    $result->free();
    $result  = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '9' %s", $user, $qtag));
    dberror($result);
    $needinfo = $result->numRows();
    $needlist = "";
    while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
    {
        $needlist = sprintf("%s <a href=\"query.php?bug=%s\">%s</a> ",$needlist, $row['bid'], $row['bid']);
    }
    $result->free();
         
    $total   = $new + $assigned + $verified + $unconfirmed + $reopened + $needinfo;
    $retdata = NULL;

    if ($total > 0)
    {
        $retdata = <<<EOT
            <tr>
                <td width="20%" valign="top">{$name}</td>
                <td width="15%" align="center" valign="top">{$newlist}&nbsp;</td>
                <td width="15%" align="center" valign="top">{$asslist}&nbsp;</td>
                <td width="15%" align="center" valign="top">{$verlist}&nbsp;</td>
                <td width="10%" align="center" valign="top">{$unconlist}&nbsp;</td>
                <td width="10%" align="center" valign="top">{$reoplist}&nbsp;</td>
                <td width="15%" align="center" valign="top">{$needlist}&nbsp;</td>
            </tr>
EOT;
    }
    return($retdata);
}

/**
 * Function to build report rows for 'Closed Bug Count by Owner'
 *
 * @param $user the uid of the user to report on
 * @param $name the name of the user to report on
 * @param $qtag as defined in reports
 */
function d_repclosedbugs($user, $name, $qtag)
{
    global $db;

    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '4' %s", $user, $qtag));
    dberror($result);
    $res     = $result->numRows();
    $result->free();
    $result = $db->query(sprintf("SELECT bid FROM bugs WHERE assigned = '%s' AND status = '6' %s", $user, $qtag));
    dberror($result);
    $clo     = $result->numRows();
    $result->free();
         
    $tot     = $res + $clo;
    $retdata = NULL; 

    if ($tot > 0)
    {
        $retdata = <<<EOT
            <tr>
                <td width="20%">{$name}</td>
                <td width="10%" align="center">{$res}</td>
                <td width="10%" align="center">{$clo}</td>
                <td width="10%" align="center">{$tot}</td>
            </tr>
EOT;

    } 
    return($retdata);
}

/**
 * Function to add bugactivity
 *
 * @param $bug the id of the bug to associate with
 * @param $user the uid of the user making the change
 * @param $what the activity being made
 * @param $oldval old value to set; "none" is no change to either old or new values
 * @param $newval new value to set;
 */
function do_bugactivity($bug, $user, $what, $oldval = NULL, $newval = NULL)
{
    global $db;

    // The activity codes are as follows:
    //
    // 1  - Status Change
    // 2  - Reassigned
    // 3  - State Change
    // 4  - Added to CC
    // 5  - Created Attachment
    // 6  - Resolution Priority Change
    // 7  - Severity Change
    // 8  - Description Change
    // 9  - Version Change
    // 10 - Component Change

    $date   = date("YmdHis");
    $id     = $db->nextID('bugactivity');

    if ($oldval == NULL)
    {
        $query  = "INSERT INTO bugactivity (aid, bid, b_user, b_date, what) ";
        $query .= "VALUES ('$id', '$bug', '$user', '$date', '$what')";
    } else {
        $query  = "INSERT INTO bugactivity (aid, bid, b_user, b_date, what, oldval, newval) ";
        $query .= "VALUES ('$id', '$bug', '$user', '$date', '$what', '$oldval', '$newval')";
    }

    $result = $db->query($query);
    dberror($result);
}

?>
